@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2007,2008 Jaap Haitsma <jaap@haitsma.org>
  * Copyright (C) 2007,2008 daniel g. siegel <dgsiegel@gmail.com>
+ * Copyright (C) 2008 Ryan Zeigler <zeiglerr@gmail.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -30,24 +31,22 @@
 #include <gst/gst.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <X11/Xlib.h>
-#include <X11/extensions/xf86vmode.h>
 #include <libhal.h>
 
 #include "cheese-webcam.h"
+#include "cheese-flash.h"
 
 G_DEFINE_TYPE (CheeseWebcam, cheese_webcam, G_TYPE_OBJECT)
 
 #define CHEESE_WEBCAM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CHEESE_TYPE_WEBCAM, CheeseWebcamPrivate))
 
-typedef struct
-{
-  char *video_device; 
-  char *gstreamer_src;
-  char *product_name; 
-  int num_video_formats;
-  GArray *video_formats;
-} CheeseWebcamDevice;
+#define CHEESE_WEBCAM_ERROR cheese_webcam_error_quark ()
 
+enum CheeseWebcamError
+{
+  CHEESE_WEBCAM_ERROR_UNKNOWN,
+  CHEESE_WEBCAM_ERROR_ELEMENT_NOT_FOUND
+};
 
 typedef struct
 {
@@ -55,8 +54,7 @@ typedef struct
 
   GstElement *pipeline; 
   GstBus     *bus;
-  /* We build the active pipeline by linking the appropriate pipelines listed
-     below*/
+  /* We build the active pipeline by linking the appropriate pipelines listed below*/
   GstElement *webcam_source_bin;
   GstElement *video_display_bin;
   GstElement *photo_save_bin;
@@ -77,9 +75,6 @@ typedef struct
   gboolean is_recording;
   gboolean pipeline_is_playing;
   char *photo_filename;
-    
-  XF86VidModeGamma normal_gamma;
-  float flash_intensity;
 
   int num_webcam_devices;
   char *device_name;
@@ -89,6 +84,8 @@ typedef struct
   int selected_device;
   CheeseVideoFormat *current_format;
   GHashTable *supported_resolutions;
+
+  CheeseFlash *flash;
 } CheeseWebcamPrivate;
 
 enum 
@@ -141,7 +138,11 @@ static const EffectToPipelineDesc EFFECT_TO_PIPELINE_DESC [] =
 
 static const int NUM_EFFECTS = G_N_ELEMENTS (EFFECT_TO_PIPELINE_DESC);
 
-
+GQuark
+cheese_webcam_error_quark (void)
+{
+  return g_quark_from_static_string ("cheese-webcam-error-quark");
+}
 
 static void
 cheese_webcam_set_x_overlay (CheeseWebcam *webcam)
@@ -207,6 +208,8 @@ cheese_webcam_photo_data_cb (GstElement *element, GstBuffer *buffer,
 
   g_signal_handler_disconnect (G_OBJECT(priv->photo_sink), 
                                priv->photo_handler_signal_id);
+
+  g_signal_emit (webcam, webcam_signals[PHOTO_SAVED], 0);
 }
 
 static void
@@ -216,10 +219,9 @@ cheese_webcam_bus_message_cb (GstBus *bus, GstMessage *message, CheeseWebcam *we
 
   if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_EOS)
   {
+    g_print ("Received EOS message\n");
     g_signal_emit (webcam, webcam_signals [VIDEO_SAVED], 0);
-
-    gst_element_set_locked_state (priv->video_source, FALSE);
-    gst_element_set_locked_state (priv->audio_source, FALSE);
+    
     cheese_webcam_change_sink (webcam, priv->video_display_bin, 
                                priv->photo_save_bin, priv->video_save_bin);
     priv->is_recording = FALSE;
@@ -296,6 +298,7 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
   {
     priv->webcam_devices[i].num_video_formats = 0;
     priv->webcam_devices[i].video_formats = g_array_new (FALSE, FALSE, sizeof (CheeseVideoFormat));
+    priv->webcam_devices[i].hal_udi = g_strdup (udis[i]);
   }
 
   for (i = 0; i < priv->num_webcam_devices; i++) 
@@ -725,8 +728,14 @@ fallback:
   return TRUE;
 }
 
+static void 
+cheese_webcam_set_error_element_not_found (GError **error, const char *factoryname)
+{
+  g_set_error (error, CHEESE_WEBCAM_ERROR, CHEESE_WEBCAM_ERROR_ELEMENT_NOT_FOUND, "The element '%s' could not be found.", factoryname);
+}
+
 static gboolean
-cheese_webcam_create_video_display_bin (CheeseWebcam *webcam)
+cheese_webcam_create_video_display_bin (CheeseWebcam *webcam, GError **error)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
   GstElement *tee, *video_display_queue, *video_scale, *video_sink, *save_queue;
@@ -738,20 +747,49 @@ cheese_webcam_create_video_display_bin (CheeseWebcam *webcam)
 
   cheese_webcam_create_webcam_source_bin (webcam);
 
-  priv->effect_filter = gst_element_factory_make ("identity", "effect");
-  priv->csp_post_effect = gst_element_factory_make ("ffmpegcolorspace", "csp_post_effect");
+  if ((priv->effect_filter = gst_element_factory_make ("identity", "effect")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "identity");
+    return FALSE;
+  }
+  if ((priv->csp_post_effect = gst_element_factory_make ("ffmpegcolorspace", "csp_post_effect")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "ffmpegcolorspace");
+    return FALSE;
+  }
 
-  tee = gst_element_factory_make ("tee", "tee");
+  if ((tee = gst_element_factory_make ("tee", "tee")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "tee");
+    return FALSE;
+  }
   
-  save_queue = gst_element_factory_make ("queue", "save_queue");
+  if ((save_queue = gst_element_factory_make ("queue", "save_queue")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "queue");
+    return FALSE;
+  }
 
-  video_display_queue = gst_element_factory_make ("queue", "video_display_queue");
+  if ((video_display_queue = gst_element_factory_make ("queue", "video_display_queue")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "queue");
+    return FALSE;
+  }
+
+  if ((video_scale = gst_element_factory_make ("videoscale", "video_scale")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "videoscale");
+    return FALSE;
+  }
 
-  video_scale = gst_element_factory_make ("videoscale", "video_scale");
   /* Use bilinear scaling */
   g_object_set (video_scale, "method", 1, NULL);
 
-  video_sink = gst_element_factory_make ("gconfvideosink", "video_sink");
+  if ((video_sink = gst_element_factory_make ("gconfvideosink", "video_sink")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "gconfvideosink");
+    return FALSE;
+  }
 
 
   gst_bin_add_many (GST_BIN (priv->video_display_bin), priv->webcam_source_bin, 
@@ -777,7 +815,7 @@ cheese_webcam_create_video_display_bin (CheeseWebcam *webcam)
 }
 
 static gboolean
-cheese_webcam_create_photo_save_bin (CheeseWebcam *webcam)
+cheese_webcam_create_photo_save_bin (CheeseWebcam *webcam, GError **error)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
   GstElement *csp_photo_save_bin;
@@ -788,8 +826,16 @@ cheese_webcam_create_photo_save_bin (CheeseWebcam *webcam)
 
   priv->photo_save_bin = gst_bin_new ("photo_save_bin");
 
-  csp_photo_save_bin = gst_element_factory_make ("ffmpegcolorspace", "csp_photo_save_bin");
-  priv->photo_sink = gst_element_factory_make ("fakesink", "photo_sink");
+  if ((csp_photo_save_bin = gst_element_factory_make ("ffmpegcolorspace", "csp_photo_save_bin")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "ffmpegcolorspace");
+    return FALSE;
+  }
+  if ((priv->photo_sink = gst_element_factory_make ("fakesink", "photo_sink")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "fakesink");
+    return FALSE;
+  }
 
   gst_bin_add_many (GST_BIN (priv->photo_save_bin), csp_photo_save_bin,
                     priv->photo_sink, NULL);
@@ -815,7 +861,7 @@ cheese_webcam_create_photo_save_bin (CheeseWebcam *webcam)
 }
 
 static gboolean 
-cheese_webcam_create_video_save_bin (CheeseWebcam *webcam)
+cheese_webcam_create_video_save_bin (CheeseWebcam *webcam, GError **error)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
@@ -827,20 +873,56 @@ cheese_webcam_create_video_save_bin (CheeseWebcam *webcam)
 
   priv->video_save_bin = gst_bin_new ("video_save_bin");
 
-  priv->audio_source = gst_element_factory_make ("gconfaudiosrc", "audio_source");
-  audio_queue = gst_element_factory_make ("queue", "audio_queue");
-  audio_convert = gst_element_factory_make ("audioconvert", "audio_convert");
-  audio_enc = gst_element_factory_make ("vorbisenc", "audio_enc");
+  if ((priv->audio_source = gst_element_factory_make ("gconfaudiosrc", "audio_source")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "gconfaudiosrc");
+    return FALSE;
+  }
+  if ((audio_queue = gst_element_factory_make ("queue", "audio_queue")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "queue");
+    return FALSE;
+  }
+  if ((audio_convert = gst_element_factory_make ("audioconvert", "audio_convert")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "audioconvert");
+    return FALSE;
+  }
+  if ((audio_enc = gst_element_factory_make ("vorbisenc", "audio_enc")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "vorbisenc"); 
+    return FALSE;
+  }
 
-  video_save_csp = gst_element_factory_make ("ffmpegcolorspace", "video_save_csp");
-  video_enc = gst_element_factory_make ("theoraenc", "video_enc");
+  if ((video_save_csp = gst_element_factory_make ("ffmpegcolorspace", "video_save_csp")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "ffmpegcolorspace");
+    return FALSE;
+  }
+  if ((video_enc = gst_element_factory_make ("theoraenc", "video_enc")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "theoraenc");
+    return FALSE;
+  }
   g_object_set (video_enc, "keyframe-force", 1, NULL);
-  video_save_scale = gst_element_factory_make ("videoscale", "video_save_scale");
+  if ((video_save_scale = gst_element_factory_make ("videoscale", "video_save_scale")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "videoscale");
+    return FALSE;
+  }
   /* Use bilinear scaling */
   g_object_set (video_save_scale, "method", 1, NULL);
 
-  mux = gst_element_factory_make ("oggmux", "mux");
-  priv->video_file_sink = gst_element_factory_make ("filesink", "video_file_sink");
+  if ((mux = gst_element_factory_make ("oggmux", "mux")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "oggmux");
+    return FALSE;
+  }
+  if ((priv->video_file_sink = gst_element_factory_make ("filesink", "video_file_sink")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "filesink");
+    return FALSE;
+  }
 
   gst_bin_add_many (GST_BIN (priv->video_save_bin), priv->audio_source, audio_queue,
                     audio_convert, audio_enc, video_save_csp, video_save_scale, video_enc, 
@@ -865,47 +947,6 @@ cheese_webcam_create_video_save_bin (CheeseWebcam *webcam)
   return TRUE;
 }
 
-static void
-cheese_webcam_flash_set_intensity (CheeseWebcam *webcam, float intensity)
-{
-  const float MAX_GAMMA = 10.0;
-  XF86VidModeGamma gamma;
-  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
-
-  g_return_if_fail ((intensity >= 0.0) || (intensity <= 1.0));
-
-  gamma.red = MAX_GAMMA * intensity + priv->normal_gamma.red * (1.0 - intensity);
-  gamma.green = MAX_GAMMA * intensity + priv->normal_gamma.green * (1.0 - intensity);
-  gamma.blue = MAX_GAMMA * intensity + priv->normal_gamma.blue * (1.0 - intensity);
-
-  XF86VidModeSetGamma (GDK_DISPLAY (), 0, &gamma);
-
-  priv->flash_intensity = intensity;
-}
-
-static gboolean
-cheese_webcam_flash_dim_cb (CheeseWebcam *webcam)
-{
-  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
-
-  priv->flash_intensity -= 0.1;
-  cheese_webcam_flash_set_intensity (webcam, priv->flash_intensity);
-
-  if (priv->flash_intensity <= 0.0)
-  {
-    g_signal_emit (webcam, webcam_signals [PHOTO_SAVED], 0);
-    return FALSE;
-  }
-  return TRUE;
-}
-
-static void
-cheese_webcam_flash (CheeseWebcam *webcam)
-{
-  cheese_webcam_flash_set_intensity (webcam, 1.0);
-  g_timeout_add (50, (GSourceFunc) cheese_webcam_flash_dim_cb, webcam);
-}
-
 int
 cheese_webcam_get_num_webcam_devices (CheeseWebcam *webcam)
 {
@@ -913,6 +954,63 @@ cheese_webcam_get_num_webcam_devices (CheeseWebcam *webcam)
   return priv->num_webcam_devices;
 }
 
+gboolean     
+cheese_webcam_switch_webcam_device (CheeseWebcam *webcam)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  gboolean was_recording = FALSE;
+  gboolean pipeline_was_playing = FALSE; 
+  gboolean disp_bin_created = FALSE;
+  gboolean disp_bin_added = FALSE;
+  gboolean disp_bin_linked = FALSE;
+  GError *error = NULL;
+    
+  if (priv->is_recording)
+  {
+    cheese_webcam_stop_video_recording (webcam);
+    was_recording = TRUE;
+  }
+  
+  if (priv->pipeline_is_playing)
+  {
+    cheese_webcam_stop (webcam);
+    pipeline_was_playing = TRUE;
+  }
+  
+  gst_bin_remove (GST_BIN (priv->pipeline), priv->video_display_bin);
+  
+  disp_bin_created = cheese_webcam_create_video_display_bin (webcam, &error);
+  if (!disp_bin_created)
+  {
+    return FALSE;
+  }
+  disp_bin_added = gst_bin_add (GST_BIN (priv->pipeline), priv->video_display_bin);
+  if (!disp_bin_added) 
+  {
+    gst_object_sink (priv->video_display_bin);
+    return FALSE;
+  }
+  
+  disp_bin_linked = gst_element_link (priv->video_display_bin, priv->photo_save_bin);
+  if (!disp_bin_linked)
+  {
+    gst_bin_remove (GST_BIN (priv->pipeline), priv->video_display_bin);
+    return FALSE;
+  }
+  
+  if (pipeline_was_playing)
+  {
+    cheese_webcam_play (webcam);
+  }
+  
+  /* if (was_recording)
+   {
+     Restart recording... ?
+   } */
+  
+  return TRUE;
+}
+
 void
 cheese_webcam_play (CheeseWebcam *webcam)
 {
@@ -1019,10 +1117,8 @@ cheese_webcam_stop_video_recording (CheeseWebcam *webcam)
   CheeseWebcamPrivate* priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
   /* Send EOS message down the pipeline by stopping video and audio source*/
-  gst_element_set_state (priv->video_source, GST_STATE_NULL);
-  gst_element_set_locked_state (priv->video_source, TRUE);
-  gst_element_set_state (priv->audio_source, GST_STATE_NULL);
-  gst_element_set_locked_state (priv->audio_source, TRUE);
+  gst_element_send_event (priv->video_source, gst_event_new_eos ());
+  gst_element_send_event (priv->audio_source, gst_event_new_eos ());
 }
 
 void
@@ -1036,10 +1132,9 @@ cheese_webcam_take_photo (CheeseWebcam *webcam, char *filename)
                                                     "handoff",
                                                     G_CALLBACK (cheese_webcam_photo_data_cb), 
                                                     webcam);
-  cheese_webcam_flash (webcam);
+  cheese_flash_fire (priv->flash);
 }
 
-
 static void
 cheese_webcam_finalize (GObject *object)
 {
@@ -1068,6 +1163,7 @@ cheese_webcam_finalize (GObject *object)
       g_free (g_array_index (priv->webcam_devices[i].video_formats, CheeseVideoFormat, j).framerates);
       g_free (g_array_index (priv->webcam_devices[i].video_formats, CheeseVideoFormat, j).mimetype);
     }
+    g_free (priv->webcam_devices[i].hal_udi);
     g_array_free (priv->webcam_devices[i].video_formats, TRUE);
   }
   g_free (priv->webcam_devices);
@@ -1210,6 +1306,8 @@ cheese_webcam_init (CheeseWebcam *webcam)
   priv->supported_resolutions = g_hash_table_new_full (g_str_hash, 
                                                        g_str_equal,
                                                        g_free, NULL);
+
+  priv->flash = cheese_flash_new ();
 }
 
 CheeseWebcam*
@@ -1236,15 +1334,39 @@ cheese_webcam_new (GtkWidget* video_window, char *webcam_device_name,
 }
 
 void
-cheese_webcam_setup (CheeseWebcam *webcam)
+cheese_webcam_setup (CheeseWebcam *webcam, char *hal_dev_udi, GError **error)
 {
   CheeseWebcamPrivate* priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
   gboolean ok = TRUE;
+  GError *tmp_error = NULL;
 
   cheese_webcam_detect_webcam_devices (webcam);
-  cheese_webcam_create_video_display_bin (webcam);
-  cheese_webcam_create_photo_save_bin (webcam);
-  cheese_webcam_create_video_save_bin (webcam);
+  
+  if (hal_dev_udi != NULL)
+  {
+    cheese_webcam_set_device_by_dev_udi (webcam, hal_dev_udi);
+  }
+
+  cheese_webcam_create_video_display_bin (webcam, &tmp_error);
+  if (tmp_error != NULL)
+  {
+    g_propagate_error (error, tmp_error);
+    return;
+  }
+
+  cheese_webcam_create_photo_save_bin (webcam, &tmp_error);
+  if (tmp_error != NULL)
+  {
+    g_propagate_error (error, tmp_error);
+    return;
+  }
+
+  cheese_webcam_create_video_save_bin (webcam, &tmp_error);
+  if (tmp_error != NULL)
+  {
+    g_propagate_error (error, tmp_error);
+    return;
+  }
 
   priv->pipeline = gst_pipeline_new ("pipeline");
 
@@ -1262,9 +1384,50 @@ cheese_webcam_setup (CheeseWebcam *webcam)
   if (!ok)
     g_error ("Unable link pipeline for photo");
 
-  gdk_threads_enter();
-  XF86VidModeGetGamma (GDK_DISPLAY (), 0, &(priv->normal_gamma));
-  gdk_threads_leave();
+}
+
+int         
+cheese_webcam_get_selected_device_index (CheeseWebcam *webcam)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  return priv->selected_device;
+}
+
+GArray *
+cheese_webcam_get_webcam_devices (CheeseWebcam *webcam)
+{
+  GArray *devices_arr;
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+
+  devices_arr = g_array_sized_new (FALSE,
+                                   TRUE,
+                                   sizeof(CheeseWebcamDevice),
+                                   priv->num_webcam_devices);
+  devices_arr = g_array_append_vals (devices_arr,
+                                     priv->webcam_devices,
+                                     priv->num_webcam_devices);
+  return devices_arr;
+}
+
+void
+cheese_webcam_set_device_by_dev_file (CheeseWebcam *webcam, char *file)
+{
+  g_object_set(webcam, "device_name", file, NULL);
+}
+
+void
+cheese_webcam_set_device_by_dev_udi (CheeseWebcam *webcam, char *udi)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  int i;
+  for (i = 0; i < priv->num_webcam_devices; i++)
+  {
+    if (strcmp(priv->webcam_devices[i].hal_udi, udi) == 0)
+    {
+      g_object_set(webcam, "device_name", priv->webcam_devices[i].video_device, NULL);
+      break;
+    }
+  }
 }
 
 GArray *
@@ -1309,4 +1472,3 @@ cheese_webcam_get_current_video_format (CheeseWebcam *webcam)
     
   return priv->current_format;
 }
-
