@@ -2,6 +2,7 @@
  * Copyright (C) 2007,2008 daniel g. siegel <dgsiegel@gmail.com>
  * Copyright (C) 2007,2008 Jaap Haitsma <jaap@haitsma.org>
  * Copyright (C) 2008 Patryk Zawadzki <patrys@pld-linux.org>
+ * Copyright (C) 2008 Ryan Zeigler <zeiglerr@gmail.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -35,6 +36,10 @@
 #include <gtk/gtk.h>
 #include <libebook/e-book.h>
 
+#ifdef HILDON
+#include <hildon/hildon-program.h>
+#endif
+
 #include "cheese-countdown.h"
 #include "cheese-effect-chooser.h"
 #include "cheese-fileutil.h"
@@ -43,7 +48,9 @@
 #include "cheese-window.h"
 #include "ephy-spinner.h"
 #include "gst-audio-play.h"
-
+#include "gedit-message-area.h"
+#include "cheese-no-camera.h"
+#include "cheese-prefs-dialog.h"
 
 #define SHUTTER_SOUNDS 5
 
@@ -53,11 +60,12 @@ typedef enum
   WEBCAM_MODE_VIDEO
 } WebcamMode;
 
-
 typedef struct 
 {
   gboolean recording;
 
+  /* UDI device requested on the command line */
+  char *startup_hal_dev_udi;
   char *video_filename;
   char *photo_filename;
 
@@ -66,12 +74,14 @@ typedef struct
   CheeseWebcam *webcam;
   WebcamMode webcam_mode;
   CheeseGConf *gconf;
+  CheeseFileUtil *fileutil;
 
   GtkWidget *window;
   GtkWidget *notebook;
   GtkWidget *notebook_bar;
 
   GtkWidget *main_vbox;
+  GtkWidget *video_vbox;
 
   GtkWidget *effect_frame;
   GtkWidget *effect_chooser;
@@ -79,6 +89,8 @@ typedef struct
   GtkWidget *throbber;
   GtkWidget *countdown_frame;
   GtkWidget *countdown;
+  GtkWidget *message_area_frame;
+  GtkWidget *message_area;
 
   GtkWidget *button_effects;
   GtkWidget *button_photo;
@@ -97,16 +109,24 @@ typedef struct
   GtkWidget *screen;
   GtkWidget *take_picture;
 
-  GtkActionGroup *actions_main;
-  GtkActionGroup *actions_toggle;
-  GtkActionGroup *actions_effects;
-  GtkActionGroup *actions_file;
-  GtkActionGroup *actions_photo;
-  GtkActionGroup *actions_video;
+#ifdef HILDON
+   GtkWidget *main_hbox;
+   GtkWidget *subwindow;
+#endif      
+
   GtkActionGroup *actions_account_photo;
-  GtkActionGroup *actions_mail;
-  GtkActionGroup *actions_fspot;
+  GtkActionGroup *actions_countdown;
+  GtkActionGroup *actions_effects;
+  GtkActionGroup *actions_preferences;
+  GtkActionGroup *actions_file;
+  GtkActionGroup *actions_sendto;
   GtkActionGroup *actions_flickr;
+  GtkActionGroup *actions_fspot;
+  GtkActionGroup *actions_mail;
+  GtkActionGroup *actions_main;
+  GtkActionGroup *actions_photo;
+  GtkActionGroup *actions_toggle;
+  GtkActionGroup *actions_video;
 
   GtkUIManager *ui_manager;
 
@@ -210,15 +230,20 @@ cheese_window_cmd_close (GtkWidget *widget, CheeseWindow *cheese_window)
   g_free (cheese_window->rand);
   g_object_unref (cheese_window->webcam);
   g_object_unref (cheese_window->actions_main);
-  g_object_unref (cheese_window->actions_photo);
-  g_object_unref (cheese_window->actions_mail);
-  g_object_unref (cheese_window->actions_toggle);
+  g_object_unref (cheese_window->actions_account_photo);
+  g_object_unref (cheese_window->actions_countdown);
   g_object_unref (cheese_window->actions_effects);
   g_object_unref (cheese_window->actions_file);
-  g_object_unref (cheese_window->actions_video);
-  g_object_unref (cheese_window->actions_account_photo);
-  g_object_unref (cheese_window->actions_fspot);
+  g_object_unref (cheese_window->actions_sendto);
   g_object_unref (cheese_window->actions_flickr);
+  g_object_unref (cheese_window->actions_fspot);
+  g_object_unref (cheese_window->actions_mail);
+  g_object_unref (cheese_window->actions_photo);
+  g_object_unref (cheese_window->actions_toggle);
+  g_object_unref (cheese_window->actions_effects);
+  g_object_unref (cheese_window->actions_preferences);
+  g_object_unref (cheese_window->actions_file);
+  g_object_unref (cheese_window->actions_video);
   g_object_unref (cheese_window->ui_manager);
   g_object_unref (cheese_window->gconf);
 
@@ -309,7 +334,7 @@ cheese_window_cmd_save_as (GtkWidget *widget, CheeseWindow *cheese_window)
                                     GTK_DIALOG_MODAL |
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                                    header);
+                                    "%s", header);
       gtk_dialog_run (GTK_DIALOG (dlg));
       gtk_widget_destroy (dlg);
       g_free (header);
@@ -338,9 +363,9 @@ cheese_window_cmd_move_file_to_trash (CheeseWindow *cheese_window, GList *files)
 
       error_dialog = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
                                              GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                             GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, primary);
+                                             GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", primary);
       gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
-                                                secondary);
+                                                "%s", secondary);
       gtk_dialog_run (GTK_DIALOG (error_dialog));
       gtk_widget_destroy (error_dialog);
 
@@ -361,8 +386,8 @@ cheese_window_move_all_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_w
   char *filename;
   GFile *file;
   GList *files_list = NULL;
-  GDir *dir;
-  char *path;
+  GDir *dir_videos, *dir_photos;
+  char *path_videos, *path_photos;
   const char *name;
 
   prompt = g_strdup_printf (_("Really move all photos and videos to the trash?"));
@@ -384,14 +409,14 @@ cheese_window_move_all_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_w
   if (response !=  GTK_RESPONSE_OK)
     return;
 
-  path = cheese_fileutil_get_media_path ();
-  dir = g_dir_open (path, 0, NULL);
-  while ((name = g_dir_read_name (dir)) != NULL)
+  //delete all videos
+  path_videos = cheese_fileutil_get_video_path (cheese_window->fileutil);
+  dir_videos = g_dir_open (path_videos, 0, NULL);
+  while ((name = g_dir_read_name (dir_videos)) != NULL)
   {
-    if (g_str_has_suffix (name, PHOTO_NAME_SUFFIX) || 
-        g_str_has_suffix (name, VIDEO_NAME_SUFFIX))
+    if (g_str_has_suffix (name, VIDEO_NAME_SUFFIX))
     {
-      filename = g_strjoin (G_DIR_SEPARATOR_S, path, name, NULL);
+      filename = g_strjoin (G_DIR_SEPARATOR_S, path_videos, name, NULL);
       file = g_file_new_for_path (filename);
 
       files_list = g_list_append (files_list, file);
@@ -400,7 +425,25 @@ cheese_window_move_all_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_w
   }
   cheese_window_cmd_move_file_to_trash (cheese_window, files_list);
   g_list_free (files_list);
-  g_dir_close(dir);
+  g_dir_close (dir_videos);
+  
+  //delete all photos
+  path_photos = cheese_fileutil_get_photo_path (cheese_window->fileutil);
+  dir_photos = g_dir_open (path_photos, 0, NULL);
+  while ((name = g_dir_read_name (dir_photos)) != NULL)
+  {
+    if (g_str_has_suffix (name, PHOTO_NAME_SUFFIX))
+    {
+      filename = g_strjoin (G_DIR_SEPARATOR_S, path_photos, name, NULL);
+      file = g_file_new_for_path (filename);
+
+      files_list = g_list_append (files_list, file);
+      g_free (filename);
+    }
+  }
+  cheese_window_cmd_move_file_to_trash (cheese_window, files_list);
+  g_list_free (files_list);
+  g_dir_close (dir_photos);
 }
 
 static void
@@ -413,14 +456,25 @@ cheese_window_move_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_windo
   filename = cheese_thumb_view_get_selected_image (CHEESE_THUMB_VIEW (cheese_window->thumb_view));
   g_return_if_fail (filename);
 
+  // return if the file does not exist, i.e. was already deleted
+  if (!g_file_test (filename, G_FILE_TEST_EXISTS)) 
+    return;
+
   file = g_file_new_for_path (filename);
-  g_free(filename);
+  g_free (filename);
 
   files_list = g_list_append (files_list, file);
   cheese_window_cmd_move_file_to_trash (cheese_window, files_list);
   g_list_free (files_list);
 }
 
+static void
+cheese_window_set_countdown (GtkWidget *widget, CheeseWindow *cheese_window)
+{
+  gboolean countdown = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (widget));
+  g_object_set (cheese_window->gconf, "gconf_prop_countdown", countdown, NULL);
+}
+
 static void
 cheese_window_cmd_set_about_me_photo (GtkWidget *widget, CheeseWindow *cheese_window)
 {
@@ -464,8 +518,8 @@ cheese_window_cmd_set_about_me_photo (GtkWidget *widget, CheeseWindow *cheese_wi
         header = g_strdup_printf (_("Could not set the Account Photo"));
         dlg = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
                                       GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, header);
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), error->message);
+                                      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", header);
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), "%s", error->message);
         gtk_dialog_run (GTK_DIALOG (dlg));
         gtk_widget_destroy (dlg);
         g_free (header);
@@ -473,7 +527,7 @@ cheese_window_cmd_set_about_me_photo (GtkWidget *widget, CheeseWindow *cheese_wi
       }
       g_free (*data);
     }
-    g_free(filename);
+    g_free (filename);
     g_object_unref (pixbuf);
   }
 }
@@ -497,6 +551,10 @@ cheese_window_cmd_command_line (GtkAction *action, CheeseWindow *cheese_window)
                                     basename, filename);
     g_free (basename);
   }
+  if (strcmp (action_name, "SendTo") == 0)
+  {
+    command_line = g_strdup_printf ("nautilus-sendto %s", filename);
+  }
   else if (strcmp (action_name, "ExportToFSpot") == 0)
   {
     char *dirname =  g_path_get_dirname (filename);
@@ -537,9 +595,9 @@ cheese_window_cmd_help_contents (GtkAction *action, CheeseWindow *cheese_window)
                                 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
                                 _("Unable to open help file for Cheese"));
-    gtk_dialog_run(GTK_DIALOG(d));
-    gtk_widget_destroy(d);
-    g_error_free(error);
+    gtk_dialog_run (GTK_DIALOG (d));
+    gtk_widget_destroy (d);
+    g_error_free (error);
   }
 }
 
@@ -575,7 +633,7 @@ cheese_window_cmd_about (GtkAction *action, CheeseWindow *cheese_window)
   gtk_show_about_dialog (GTK_WINDOW (cheese_window->window),
                          "version", VERSION,
                          "copyright", "Copyright \xc2\xa9 2007, 2008\n daniel g. siegel <dgsiegel@gmail.com>",
-                         "comments", _("A cheesy program to take photos and videos from your webcam"),
+                         "comments", _("Take photos and videos with your webcam, with fun graphical effects"),
                          "authors", authors,
                          "translator-credits", translators,
                          "website", "http://www.gnome.org/projects/cheese",
@@ -677,7 +735,6 @@ cheese_window_effect_button_pressed_cb (GtkWidget *widget, CheeseWindow *cheese_
   else
   {
     gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook), 1);
-    gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_effects), _("_Back"));
     gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture), FALSE);
     gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
     gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
@@ -759,7 +816,7 @@ cheese_window_photo_video_toggle_buttons_cb (GtkWidget *widget, CheeseWindow *ch
   {
     cheese_window->webcam_mode = WEBCAM_MODE_PHOTO;
 
-    str = g_strconcat ("<b>", _("_Take a photo"), "</b>", NULL);
+    str = g_strconcat ("<b>", _("_Take a Photo"), "</b>", NULL);
     gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
     g_free (str);
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
@@ -791,16 +848,40 @@ cheese_window_countdown_picture_cb (gpointer data)
   audio_play = gst_audio_play_file (file, &error);
   if (!audio_play) 
   {
-    g_warning (error ? error->message : "Unknown error");
+    g_warning ("%s", error ? error->message : "Unknown error");
     g_error_free (error);
   }
 
   g_free (file);
 
-  cheese_window->photo_filename = cheese_fileutil_get_new_media_filename (WEBCAM_MODE_PHOTO);
+  cheese_window->photo_filename = cheese_fileutil_get_new_media_filename (cheese_window->fileutil, WEBCAM_MODE_PHOTO);
   cheese_webcam_take_photo (cheese_window->webcam, cheese_window->photo_filename);
 }
 
+static void
+cheese_window_no_camera_message_area_response (GtkWidget *widget, gint response_id, GtkWidget *cheese_window)
+{
+  GError *error = NULL;
+  gboolean ret;
+
+  if (response_id == GTK_RESPONSE_HELP)
+  {
+    ret = g_app_info_launch_default_for_uri ("ghelp:cheese?faq", NULL, &error);
+
+    if (ret == FALSE) 
+    {
+      GtkWidget *d;
+      d = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window), 
+                                  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                  GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+                                  _("Unable to open help file for Cheese"));
+      gtk_dialog_run(GTK_DIALOG(d));
+      gtk_widget_destroy(d);
+      g_error_free(error);
+    }
+  }
+}
+
 static void
 cheese_window_stop_recording (CheeseWindow *cheese_window)
 {
@@ -810,7 +891,7 @@ cheese_window_stop_recording (CheeseWindow *cheese_window)
     gtk_widget_set_sensitive (cheese_window->button_effects, TRUE);
     gtk_widget_set_sensitive (cheese_window->button_photo, TRUE);
     gtk_widget_set_sensitive (cheese_window->button_video, TRUE);
-    gchar * str = g_strconcat ("<b>", _("_Start recording"), "</b>", NULL);
+    gchar * str = g_strconcat ("<b>", _("_Start Recording"), "</b>", NULL);
     gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
     g_free (str);
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
@@ -844,6 +925,7 @@ static void
 cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window)
 {
   char *str;
+  
 
   if (cheese_window->webcam_mode == WEBCAM_MODE_PHOTO)
   {
@@ -872,13 +954,13 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
       gtk_widget_set_sensitive (cheese_window->button_effects, FALSE);
       gtk_widget_set_sensitive (cheese_window->button_photo, FALSE);
       gtk_widget_set_sensitive (cheese_window->button_video, FALSE);
-      str = g_strconcat ("<b>", _("_Stop recording"), "</b>", NULL);
+      str = g_strconcat ("<b>", _("_Stop Recording"), "</b>", NULL);
       gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
       g_free (str);
       gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
       gtk_image_set_from_stock (GTK_IMAGE (cheese_window->image_take_photo), GTK_STOCK_MEDIA_STOP, GTK_ICON_SIZE_BUTTON);
 
-      cheese_window->video_filename = cheese_fileutil_get_new_media_filename (WEBCAM_MODE_VIDEO);
+      cheese_window->video_filename = cheese_fileutil_get_new_media_filename (cheese_window->fileutil, WEBCAM_MODE_VIDEO);
       cheese_webcam_start_video_recording (cheese_window->webcam, cheese_window->video_filename);
 
       cheese_window->recording = TRUE;
@@ -890,30 +972,45 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
   }
 }
 
+static void 
+cheese_window_preferences_cb (GtkAction *action, CheeseWindow *cheese_window)
+{
+  cheese_prefs_dialog_run (cheese_window->window, cheese_window->gconf,
+                           cheese_window->webcam);
+}
+
 static const GtkActionEntry action_entries_main[] = {
   {"Cheese", NULL, N_("_Cheese")},
 
   {"Edit", NULL, N_("_Edit")},
-  {"RemoveAll", NULL, N_("Move all to Trash"), NULL, NULL, G_CALLBACK (cheese_window_move_all_media_to_trash)},
+  {"RemoveAll", NULL, N_("Move All to Trash"), NULL, NULL, G_CALLBACK (cheese_window_move_all_media_to_trash)},
 
   {"Help", NULL, N_("_Help")},
 
   {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (cheese_window_cmd_close)},
-  {"HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Help on this application"), G_CALLBACK (cheese_window_cmd_help_contents)},
+  {"HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Help on this Application"), G_CALLBACK (cheese_window_cmd_help_contents)},
   {"About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (cheese_window_cmd_about)},
 
 };
 
+static const GtkToggleActionEntry action_entries_countdown[] = {
+  {"Countdown", NULL, N_("Countdown"), NULL, NULL, G_CALLBACK (cheese_window_set_countdown), FALSE},
+};
+
 static const GtkToggleActionEntry action_entries_effects[] = {
   {"Effects", NULL, N_("_Effects"), NULL, NULL, G_CALLBACK (cheese_window_effect_button_pressed_cb), FALSE},
 };
 
+static const GtkActionEntry action_entries_preferences[] = {
+  {"Preferences", GTK_STOCK_PREFERENCES, N_("Preferences"), NULL, NULL, G_CALLBACK (cheese_window_preferences_cb)},
+};
+
 static const GtkRadioActionEntry action_entries_toggle[] = {
   {"Photo", NULL, N_("_Photo"), NULL, NULL, 0},
   {"Video", NULL, N_("_Video"), NULL, NULL, 1},
 };
 
-static const GtkActionEntry action_entries_edit_file[] = {
+static const GtkActionEntry action_entries_file[] = {
   {"Open", GTK_STOCK_OPEN, N_("_Open"), "<control>O", NULL, G_CALLBACK (cheese_window_cmd_open)},
   {"SaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), "<control>S", NULL, G_CALLBACK (cheese_window_cmd_save_as)},
   {"MoveToTrash", "user-trash", N_("Move to _Trash"), "Delete", NULL, G_CALLBACK (cheese_window_move_media_to_trash)},
@@ -935,6 +1032,10 @@ static const GtkActionEntry action_entries_mail[] = {
   {"SendByMail", NULL, N_("Send by _Mail"), NULL, NULL, G_CALLBACK (cheese_window_cmd_command_line)},
 };
 
+static const GtkActionEntry action_entries_sendto[] = {
+  {"SendTo", NULL, N_("Send _To"), NULL, NULL, G_CALLBACK (cheese_window_cmd_command_line)},
+};
+
 static const GtkActionEntry action_entries_fspot[] = {
   {"ExportToFSpot", NULL, N_("Export to F-_Spot"), NULL, NULL, G_CALLBACK (cheese_window_cmd_command_line)},
 };
@@ -1003,14 +1104,42 @@ cheese_window_radio_action_group_new (CheeseWindow *cheese_window, char *name,
   return action_group;
 }
 
+static void
+cheese_window_set_message_area (CheeseWindow *cheese_window,
+                                GtkWidget *message_area)
+{
+  if (cheese_window->message_area == message_area)
+    return;
+
+  if (cheese_window->message_area != NULL)
+    gtk_widget_destroy (cheese_window->message_area);
+
+  cheese_window->message_area = message_area;
+
+  if (message_area == NULL)
+    return;
+
+  gtk_container_add (GTK_CONTAINER (cheese_window->message_area_frame), cheese_window->message_area);
+  gtk_widget_show (GTK_WIDGET(cheese_window->message_area));
+}
+
 static void
 cheese_window_create_window (CheeseWindow *cheese_window)
 {
-  GtkWidget *menubar;
   GError *error=NULL;
   char *path;
   GtkBuilder* builder;
 
+#ifdef HILDON
+  HildonProgram *program = hildon_program_get_instance();
+  GtkWidget *menu;
+  GtkWidget *menuitem;
+#else
+  GtkWidget *menubar;
+#endif
+
+  cheese_window->message_area = NULL;
+
   builder = gtk_builder_new ();
   gtk_builder_add_from_file (builder, PACKAGE_DATADIR"/cheese.ui", &error);
 
@@ -1030,6 +1159,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->label_take_photo   = GTK_WIDGET (gtk_builder_get_object (builder, "label_take_photo"));
   cheese_window->label_video        = GTK_WIDGET (gtk_builder_get_object (builder, "label_video"));
   cheese_window->main_vbox          = GTK_WIDGET (gtk_builder_get_object (builder, "main_vbox"));
+  cheese_window->video_vbox         = GTK_WIDGET (gtk_builder_get_object (builder, "video_vbox"));
   cheese_window->notebook           = GTK_WIDGET (gtk_builder_get_object (builder, "notebook"));
   cheese_window->notebook_bar       = GTK_WIDGET (gtk_builder_get_object (builder, "notebook_bar"));
   cheese_window->screen             = GTK_WIDGET (gtk_builder_get_object (builder, "video_screen"));
@@ -1038,6 +1168,27 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->throbber_frame     = GTK_WIDGET (gtk_builder_get_object (builder, "throbber_frame"));
   cheese_window->countdown_frame    = GTK_WIDGET (gtk_builder_get_object (builder, "countdown_frame"));
   cheese_window->effect_frame       = GTK_WIDGET (gtk_builder_get_object (builder, "effect_frame"));
+  cheese_window->message_area_frame = GTK_WIDGET (gtk_builder_get_object (builder, "message_area_frame"));
+
+#ifdef HILDON
+  /* Reparent widgets in case we use hildon. This saves us maintaining two
+     GtkBuilder ui files
+  */
+  cheese_window->window = hildon_window_new();
+  cheese_window->main_hbox = gtk_hbox_new(FALSE, 0);
+  hildon_program_add_window (program, HILDON_WINDOW(cheese_window->window));
+  gtk_container_add (GTK_CONTAINER(cheese_window->window), cheese_window->main_hbox);  
+  gtk_widget_ref(cheese_window->thumb_scrollwindow);
+  gtk_widget_ref(cheese_window->video_vbox);
+  gtk_container_remove(GTK_CONTAINER(cheese_window->video_vbox), cheese_window->thumb_scrollwindow);
+  gtk_container_remove(GTK_CONTAINER(cheese_window->main_vbox),cheese_window->video_vbox);
+  gtk_box_pack_start (GTK_BOX(cheese_window->main_hbox), cheese_window->video_vbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX(cheese_window->main_hbox), GTK_WIDGET(cheese_window->thumb_scrollwindow), FALSE, FALSE, 0);
+  gtk_widget_destroy(cheese_window->main_vbox);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(cheese_window->thumb_scrollwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_widget_unref(cheese_window->thumb_scrollwindow);
+  gtk_widget_unref(cheese_window->video_vbox);
+#endif
 
   g_object_unref (builder);
 
@@ -1045,6 +1196,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
   g_free (str);
   gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
+  gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture), FALSE);
 
   cheese_window->thumb_view = cheese_thumb_view_new ();
   gtk_container_add (GTK_CONTAINER (cheese_window->thumb_scrollwindow), cheese_window->thumb_view);
@@ -1088,14 +1240,22 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                           "ActionsEffects", 
                                                                           action_entries_effects, 
                                                                           G_N_ELEMENTS (action_entries_effects));
+  cheese_window->actions_preferences = cheese_window_action_group_new (cheese_window, 
+                                                                       "ActionsPreferences", 
+                                                                       action_entries_preferences, 
+                                                                       G_N_ELEMENTS (action_entries_preferences));
   cheese_window->actions_file = cheese_window_action_group_new (cheese_window, 
                                                                 "ActionsFile", 
-                                                                action_entries_edit_file, 
-                                                                G_N_ELEMENTS (action_entries_edit_file));
+                                                                action_entries_file, 
+                                                                G_N_ELEMENTS (action_entries_file));
   cheese_window->actions_photo = cheese_window_action_group_new (cheese_window, 
                                                                  "ActionsPhoto", 
                                                                  action_entries_photo, 
                                                                  G_N_ELEMENTS (action_entries_photo));
+  cheese_window->actions_countdown = cheese_window_toggle_action_group_new (cheese_window, 
+                                                                            "ActionsCountdown", 
+                                                                            action_entries_countdown, 
+                                                                            G_N_ELEMENTS (action_entries_countdown));
   cheese_window->actions_video = cheese_window_toggle_action_group_new (cheese_window, 
                                                                         "ActionsVideo", 
                                                                         action_entries_video, 
@@ -1109,8 +1269,25 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                 "ActionsMail", 
                                                                 action_entries_mail, 
                                                                 G_N_ELEMENTS (action_entries_mail));
-  path = g_find_program_in_path ("gnome-open");
-  gtk_action_group_set_visible (cheese_window->actions_mail, path != NULL);
+  cheese_window->actions_sendto = cheese_window_action_group_new (cheese_window, 
+                                                                  "ActionsSendTo", 
+                                                                  action_entries_sendto, 
+                                                                  G_N_ELEMENTS (action_entries_sendto));
+
+  /* handling and activation of send to/send mail actions. We only show one send mail action */
+  path = g_find_program_in_path ("nautilus-sendto");
+  gboolean nautilus_sendto = (path != NULL);
+  if (nautilus_sendto)
+  {
+    gtk_action_group_set_visible (cheese_window->actions_sendto, TRUE);
+    gtk_action_group_set_visible (cheese_window->actions_mail, FALSE);
+  }
+  else
+  {
+    path = g_find_program_in_path ("gnome-open");
+    gtk_action_group_set_visible (cheese_window->actions_mail, path != NULL);
+    gtk_action_group_set_visible (cheese_window->actions_sendto, FALSE);
+  }
   g_free (path);
 
   cheese_window->actions_fspot = cheese_window_action_group_new (cheese_window, 
@@ -1138,8 +1315,33 @@ cheese_window_create_window (CheeseWindow *cheese_window)
     g_error_free (error);
   }
 
+  GtkAction *action = gtk_ui_manager_get_action (cheese_window->ui_manager, "/MainMenu/Cheese/CountdownToggle");
+  gboolean countdown;
+  g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
+  if (countdown)
+  {
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+  }
+
+#ifdef HILDON
+  menu = gtk_menu_new();
+  menuitem = gtk_menu_item_new_with_label(_("Quit"));
+  g_signal_connect(menuitem,"activate",
+                   GTK_SIGNAL_FUNC(cheese_window_cmd_close),
+                   cheese_window);
+  gtk_menu_append(menu, menuitem);
+  
+  menuitem = gtk_menu_item_new_with_label(_("About"));
+  g_signal_connect(menuitem,"activate",
+                   GTK_SIGNAL_FUNC(cheese_window_cmd_about), 
+                   cheese_window);
+  gtk_menu_append(menu, menuitem);
+  
+  hildon_window_set_menu (HILDON_WINDOW(cheese_window->window),GTK_MENU(menu));
+#else
   menubar = gtk_ui_manager_get_widget (cheese_window->ui_manager, "/MainMenu");
   gtk_box_pack_start (GTK_BOX (cheese_window->main_vbox), menubar, FALSE, FALSE, 0);
+#endif
 
   cheese_window->thumb_view_popup_menu = gtk_ui_manager_get_widget (cheese_window->ui_manager, 
                                                                     "/ThumbnailPopup");
@@ -1171,12 +1373,48 @@ void
 setup_camera (CheeseWindow *cheese_window)
 {
   char *webcam_device = NULL;
+  int x_resolution;
+  int y_resolution;
+  GtkWidget *message_area;
+  
+  GError *error;
+  g_object_get (cheese_window->gconf, "gconf_prop_x_resolution", &x_resolution,
+                "gconf_prop_y_resolution", &y_resolution, "gconf_prop_webcam", &webcam_device, NULL);
 
-  g_object_get (cheese_window->gconf, "gconf_prop_webcam", &webcam_device, NULL);
+  gdk_threads_enter ();
+  cheese_window->webcam = cheese_webcam_new (cheese_window->screen, 
+                                             webcam_device, x_resolution,
+                                             y_resolution);
+  gdk_threads_leave ();
 
-  cheese_window->webcam = cheese_webcam_new (cheese_window->screen, webcam_device);
   g_free (webcam_device);
 
+  error = NULL;
+  cheese_webcam_setup (cheese_window->webcam, cheese_window->startup_hal_dev_udi, &error);
+  if (error != NULL)
+  {
+    GtkWidget *dialog;
+
+    gdk_threads_enter ();
+
+    dialog = gtk_message_dialog_new (NULL,
+                                     0,
+                                     GTK_MESSAGE_ERROR, 
+                                     GTK_BUTTONS_OK, 
+                                     "%s", error->message);
+
+    g_error_free(error);
+
+    g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
+    gtk_window_set_title(GTK_WINDOW (dialog), "Critical Error");
+    gtk_dialog_run (GTK_DIALOG (dialog));
+
+    // Clean up and exit
+    cheese_window_cmd_close(NULL, cheese_window);
+
+    gdk_threads_leave ();
+  }
+
   g_signal_connect (cheese_window->webcam, "photo-saved",
                     G_CALLBACK (cheese_window_photo_saved_cb), cheese_window);
   g_signal_connect (cheese_window->webcam, "video-saved",
@@ -1189,19 +1427,33 @@ setup_camera (CheeseWindow *cheese_window)
   gdk_threads_enter ();
   gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook), 0);
   ephy_spinner_stop (EPHY_SPINNER (cheese_window->throbber));
+  if (cheese_webcam_get_num_webcam_devices (cheese_window->webcam) == 0)
+  {
+    message_area = cheese_no_camera_message_area();
+
+    g_signal_connect (message_area,
+                      "response",
+                      G_CALLBACK (cheese_window_no_camera_message_area_response),
+                      cheese_window);
+
+    cheese_window_set_message_area (cheese_window, message_area);
+  }
+  gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture), TRUE);
   gdk_threads_leave ();
 }
 
 void
-cheese_window_init ()
+cheese_window_init (char *hal_dev_udi)
 {
   CheeseWindow *cheese_window;
 
-  cheese_window = g_new (CheeseWindow, 1);
+  cheese_window = g_new0 (CheeseWindow, 1);
 
+  cheese_window->startup_hal_dev_udi = hal_dev_udi;
   cheese_window->gconf = cheese_gconf_new ();
   cheese_window->audio_play_counter = 0;
   cheese_window->rand = g_rand_new ();
+  cheese_window->fileutil = cheese_fileutil_new ();
 
   cheese_window_create_window (cheese_window);
  
